//---------------------------------------------------------------------------
// This routine computes and displays the upper waterfall
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "DoWF1.h"

#pragma package(smart_init)

static float val[2048];
//---------------------------------------------------------------------------
void __fastcall DoWF1(float* FF1spectrum)
{
	static float Smooth[FF1SMAX], range = 0.9, lmax;
	float  t, dB, vm, pk, fm, den, quart, vrange, base, dist;
	int    j, k, y, offs, idx, span = MainForm->WF1->Width, validx[2048], inc;
	char   st[128];
	Byte   *pixptr;
	static POINT PT[FF1SMAX];
	static bool first = true;

	if(first)
	{
	  first = false;
	  VirtualAlloc(Smooth, sizeof(float) * FF1SMAX, MEM_COMMIT, PAGE_READWRITE);
	  VirtualLock(Smooth, sizeof(float) * FF1SMAX);
	}

	if(FF1spectrum == NULL)  // then dummy call at program initialization
	  goto just_cosmetic;

	for (j=0; j < MF->ff1Size; j++)
	{
	   t = FF1spectrum[j];
	   Smooth[j] = (Smooth[j]*(MainForm->Avg1-1) + t) / MainForm->Avg1;
	}

	offs = MainForm->FF1InizioScala / MainForm->FF1HzPerPixel + 0.5;

	for (j=0; j<span; j++)
	{
	   int index = j + offs;

	   if(index < 0) index += MainForm->ff1Size;

	   t = Smooth[index];
	   t = max(t, 1.e-16F);
	   t = 20 * log10(t / MainForm->SP1MaxVal);
	   t = -min(160., fabs(t));
	   val[j] = 160. + t;        // val goes from 0 to 160

	   PT[j].x = j;
	   PT[j].y = getYfromDB(t);

	   if(!MainForm->WF1avg)
	   {
		 t = FF1spectrum[index];
		 t = max(t, 1.e-16F);
		 t = 20 * log10(t / MainForm->SP1MaxVal);
		 t = -min(160., fabs(t));
		 val[j] = 160. + t;        // val goes from 0 to 160
	   }
	}

just_cosmetic:
	FillRect(MainForm->SP1bmhdc, &MainForm->SP1rect, MainForm->SP1back);

// the RX band blue window
	FillRect(MainForm->SP1bmhdc, &MainForm->rxband, MainForm->winback);

	dB = -160;

	SelectObject(MainForm->SP1bmhdc, MainForm->darkpen);
	SetTextColor(MainForm->SP1bmhdc, MainForm->dBcolor);

	dist = getYfromDB(-160.) - getYfromDB(-150.);
	if(dist >= 90) inc = 1;
	else
	  if(dist >= 50) inc = 2;
	  else
		if(dist >= 22) inc = 5;
		else inc = 10;

	while(dB <= 0)
	{  y = getYfromDB(dB);
	   MoveToEx(MainForm->SP1bmhdc, 20, y, NULL);
	   LineTo(MainForm->SP1bmhdc, MainForm->SP1W - 1, y);
	   if(dB < 0) sprintf(st, "%d", (int)dB);
	   else       strcpy(st, "  0");
	   TextOut(MainForm->SP1bmhdc, 2, y-5, st, strlen(st));
	   if(dB == 0)
	   { strcpy(st, "dB");
		 TextOut(MainForm->SP1bmhdc, 20, y-5, st, strlen(st));
	   }
	   dB += inc;
	}

// draw the current spectrum
	SelectObject(MainForm->SP1bmhdc, MainForm->SP1pen);
	bool rc = ::Polyline(MainForm->SP1bmhdc, PT, span - 1);

// the moving red line in SP1
	if(MainForm->inSP1 && !MainForm->inBetween)
	{
	   SelectObject(MainForm->SP1bmhdc, MainForm->redpen);
	   MoveToEx(MainForm->SP1bmhdc, MainForm->mouseSP1.x, 0, NULL);
	   LineTo(MainForm->SP1bmhdc, MainForm->mouseSP1.x, MainForm->SP1H);
	}

	{
	   MainForm->FF1yellowX = (MainForm->FF1freq - MainForm->FF1InizioScala)
											/ MainForm->FF1HzPerPixel + 0.5;
	   SelectObject(MainForm->SP1bmhdc, MainForm->graypen);
	   MoveToEx(MainForm->SP1bmhdc, MainForm->FF1yellowX, 0, NULL);
	   LineTo(MainForm->SP1bmhdc, MainForm->FF1yellowX, MainForm->SP1H);
	}

	if(FF1spectrum != NULL && MainForm->PFF1->wf1time)
	{
// now compute the waterfall in WF1

	  for (j=0; j<span; j++) validx[j] = j; // for qsort

	  qsort(validx, span, sizeof(validx[0]), indexsort);
	  quart = val[validx[span/4]]; // if(quart < 80.) quart = 80.;

	  vrange = 160. - quart;

	  base = 10. * (float)MainForm->WF1Gain->Position / MainForm->WF1Gain->Max;

	  if(MainForm->revWF1scroll)
		pixptr = (Byte *)MainForm->WF1bm->ScanLine[MainForm->WF1H-1];
	  else
		pixptr = (Byte *)MainForm->WF1bm->ScanLine[0];

// if not resizing, then update scanline
	  if(!MainForm->WF1resizing && !MainForm->palettechanged)
	  {
		for(int k=0; k<span; k++)
		{
			fm  = pow(max(0.F, base * (val[k] - quart))/vrange, MainForm->wf1gamma);
			idx = fm * 255. +0.5;
			idx = min(idx, 255);
			idx = max(1, idx);
			pixptr[k] = (Byte)idx;
		}
	  }
	}

	EnterCriticalSection(&MainForm->screenUpdate);
	  HDC hdcform = GetDC(MainForm->hwnd);
	  BitBlt(hdcform, MainForm->SP1L, MainForm->SP1T,
					  MainForm->SP1W, MainForm->SP1H,
					  MainForm->SP1bmhdc, 0, 0, SRCCOPY);
	  ReleaseDC(MainForm->hwnd, hdcform);
	LeaveCriticalSection(&MainForm->screenUpdate);

	if(MainForm->inSP1 && MainForm->PFF1 != NULL) MainForm->PFF1->DoSync();
}
//---------------------------------------------------------------------------
int inline getYfromDB(float dB)
{
	return MainForm->SP1rect.Bottom  * (dB - MainForm->SP1toprange) /
			(MainForm->SP1bottomrange - MainForm->SP1toprange) + 0.5;
}
//---------------------------------------------------------------------------
static int indexsort(const void *a, const void *b)
{
	return (val[*(int *)a] < val[*(int *)b]) ? -1 : 1;
}
//----------------------------------------------------------------------

