//---------------------------------------------------------------------------
// Here the FFT, spectrum and waterfall of the lower panel are computed
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "Functions2.h"

#pragma package(smart_init)

static float  val[4096];
static double DynFacSpectr[BUFSIZE/2 + 1];
//---------------------------------------------------------------------------
void __fastcall TProcessFF2::ComputeFF2(void)         // called by ProcessFF2
{
    static int flags, iflags, ipeak, oldpolyfac = 0;
    static int order, span, indexes[FF2SMAX];
    static float range = 1.9, gamma = 0.9, Smooth[FF2SMAX/2];
	static float Spectr[FF2SMAX/2+1], dBcorr,
				  maskmag[AFFTLEN + 2], mskres[FF2SMAX];
	static POINT pt[FF2SMAX], mskpt[FF2SMAX];

	int    j, y, offs, sortBeg, sortEnd, sortSpan, idx;
	int    validx[FF2SMAX/2], inc;
	float  t, dB, vm, fm, den, fpeak, fdelta, quart, valquart,
		   efac, gain, vrange, base, dist;
	char   st[128];
	Byte   *pixptr;

	if(size != MF->ff2Size || MainForm->polyfactor != oldpolyfac)
	{
		ZeroMemory(PolyQue,  sizeof(float)*MainForm->polyfactor*FF2SMAX);
		ZeroMemory(wnuttall, sizeof(float)*MainForm->polyfactor*FF2SMAX);
		ipeak = 0;
		for(j = 0; j < FF2SMAX / 2; j++) Smooth[j] = 0.F;
		nspcFreeTwdTbls();
		nspFreeBitRevTbls();

		span   = MainForm->WF2->Width;
		size   = MF->ff2Size;
		order  = (int)(log(size)/log(2.) +0.0001);  // povero Borland....
		flags  = NSP_Forw | NSP_NoBitRev;
		iflags = NSP_Inv  | NSP_NoBitRev;
		nspdRealFft(NULL, order, (flags | NSP_Init));

		for(j = 0; j < MainForm->polyfactor*size; j++)
		{
		   float tmp = sin((float)j / (MainForm->polyfactor*size - 1) * PI);
		   sin3[j] = tmp * tmp * tmp;
		   sin5[j] = sin3[j] * tmp * tmp;

		}

// compute the Nuttall window for the new size
		for(j = 0; j < MainForm->polyfactor*size; j++)
		{
		   wnuttall[j] = 0.355768 - 0.487396 * cos(2*PI*j/(MainForm->polyfactor*size-1))
						+0.144232  * cos(4*PI*j/(MainForm->polyfactor*size-1))
						- 0.012604 * cos(6*PI*j/(MainForm->polyfactor*size-1));
		}

// compute the resampling indexes for displaying the bandpass shape
		resamp(AFFTLEN, size, indexes);

		oldpolyfac = MainForm->polyfactor;
	}

	EnterCriticalSection(&MainForm->acFF2que);
	  memcpy(PolyFF2, PolyQue, MainForm->polyfactor * size * sizeof(float));
	LeaveCriticalSection(&MainForm->acFF2que);

	switch(MainForm->wintype)
	{
	   case Hann  :
			nspsWinHann(PolyFF2, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.0593;  break;
			  case 2  : dBcorr = 0.6683;  break;
			  case 4  : dBcorr = 0.9333;  break;
			}
			break;

	   case Blackman  :
			nspsWinBlackman(PolyFF2, MainForm->polyfactor * size, -0.16);
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.2589;  break;
			  case 2  : dBcorr = 0.7413;  break;
			  case 4  : dBcorr = 0.7674;  break;
			}
			break;

	   case Hamming  :
			nspsWinHamming(PolyFF2, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.0116;  break;
			  case 2  : dBcorr = 0.6607;  break;
			  case 4  : dBcorr = 1.1614;  break;
			}
			break;

	   case wsin3  :
			nspsbMpy2(sin3, PolyFF2, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.2445;  break;
			  case 2  : dBcorr = 0.7328;  break;
			  case 4  : dBcorr = 0.7499;  break;
			}
			break;

	   case wsin5  :
			nspsbMpy2(sin5, PolyFF2, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.5311;  break;
			  case 2  : dBcorr = 0.8511;  break;
			  case 4  : dBcorr = 0.6683;  break;
			}
			break;

	   case nuttall :
			nspsbMpy2(wnuttall, PolyFF2, MainForm->polyfactor * size);
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 1.4454;  break;
			  case 2  : dBcorr = 0.8222;  break;
			  case 4  : dBcorr = 0.6918;  break;
			}
			break;

	   case nowin  :
			switch(MainForm->polyfactor)
			{
			  case 1  : dBcorr = 0.5957;  break;
			  case 2  : dBcorr = 0.5957;  break;
			  case 4  : dBcorr = 0.5957;  break;
			}
			break;
	}

						  // WOLA processing
	memcpy(FF2array, PolyFF2, size * sizeof(float)); // copy only the first chunk

	for(j = 1; j < MainForm->polyfactor; j++)
	  nspsbAdd2(PolyFF2 + j * size, FF2array, size);  // and add the remaining

	nspsRealFftNip(FF2array, FF2data, order, flags);

// compute magnitude : complex input; real output; single precision
	nspcbMag(FF2data, Spectr, size/2);

	nspsbMpy1(dBcorr, Spectr, size/2);  // window correction factor

// compute magnitude of the windowed-sync filter
	nspcbMag(MainForm->FF2mask, maskmag, AFFTLEN);

	for (j=0; j < size/2; j++)
	{
	   float f;

	   validx[j] = j;
	   t = Spectr[j];
	   Smooth[j] = (Smooth[j]*(MainForm->Avg2-1) + t) / MainForm->Avg2;
	   mskres[j] = max(maskmag[indexes[j]], 1.e-16);
	   mskres[j] = 20 * log10(mskres[j]);
	}

	offs  = MainForm->FF2InizioScala / MainForm->FF2HzPerPixel + 0.5;

	sortBeg  = (MainForm->bpLocut - MainForm->FF2InizioScala)
											  / MainForm->FF2HzPerPixel + 0.5;
	sortEnd  = (MainForm->bpHicut - MainForm->FF2InizioScala)
											  / MainForm->FF2HzPerPixel + 0.5;

	sortBeg = max(sortBeg, 0);
	sortEnd = max(sortEnd, 0);
	sortBeg = min(sortBeg, (MainForm->FF2Scale->StopVal  -
				MainForm->FF2Scale->StartVal - 1) / MainForm->FF2HzPerPixel + 0.5);
	sortEnd = min(sortEnd, (MainForm->FF2Scale->StopVal  -
				MainForm->FF2Scale->StartVal - 1) / MainForm->FF2HzPerPixel + 0.5);
	sortSpan = sortEnd - sortBeg;

	for (j=0; j<span; j++)
	{
	   int index = j + offs;

	   t = Smooth[index];

	   t = max(t, 1.e-16F);
	   t = 20 * log10(t / MainForm->SP2MaxVal);
	   t = -min(160., fabs(t));
	   val[j] = 160. + t;        // val goes from 0 to 160

	   pt[j].x = j;
	   pt[j].y = get2YfromDB(t);
	   mskpt[j].x = j;
	   mskpt[j].y = get2YfromDB(mskres[index]);

	   if(!MainForm->WF2avg)
	   {
		 t = Spectr[index];

		 t = max(t, 1.e-16F);
		 t = 20 * log10(t / MainForm->SP2MaxVal);
		 t = -min(160., fabs(t));
		 val[j] = 160. + t;        // val goes from 0 to 160
	   }
	}

	qsort(validx + sortBeg, sortSpan, sizeof(validx[0]), sortfun);

	ipeak = validx[sortBeg + sortSpan - 1] + offs; // index in Smooth of the peak
	fpeak = ipeak * MainForm->FF2HzPerPixel;       // from index to actual frequency
	fdelta = MainForm->CWPitch - fpeak;            // how far from bandpass center ?
	if(MainForm->mode == CW)
	  if(MainForm->zap)
	  {
		MainForm->FF1freq -= fdelta;
		MainForm->AdjustBlueWindow();
		MainForm->BZAPClick(NULL);
	  }
	  else
		if(MainForm->afc)
		{
		   MainForm->FF1freq -= fdelta/200.; // correct a bit more slower each time
		   MainForm->AdjustBlueWindow();
		}


	FillRect(MainForm->SP2bmhdc, &MainForm->SP2rect, MainForm->SP2back);

	dB = -160;

	SelectObject(MainForm->SP2bmhdc, MainForm->darkpen);
	SetTextColor(MainForm->SP2bmhdc, MainForm->dBcolor);
	SelectObject(MainForm->SP2bmhdc, MainForm->dBfont);

	dist = get2YfromDB(-160.) - get2YfromDB(-150.);
	if(dist >= 100) inc = 1;
	else
	  if(dist >= 70) inc = 2;
	  else
		if(dist >= 30) inc = 5;
		else inc = 10;

	while(dB <= 0)
	{  y = get2YfromDB(dB);
	   MoveToEx(MainForm->SP2bmhdc, 20, y, NULL);
	   LineTo(MainForm->SP2bmhdc, MainForm->SP2W - 1, y);
	   if(dB < 0) sprintf(st, "%d", (int)dB);
	   else       strcpy(st, "  0");
	   TextOut(MainForm->SP2bmhdc, 2, y-5, st, strlen(st));
	   if(dB == 0)
	   { strcpy(st, "dB");
		 TextOut(MainForm->SP2bmhdc, 20, y-5, st, strlen(st));
	   }
	   dB += inc;
	}

	SelectObject(MainForm->SP2bmhdc, MainForm->SP2pen);
	::Polyline(MainForm->SP2bmhdc, pt, span - 1);
	SelectObject(MainForm->SP2bmhdc, MainForm->redpen);
	::Polyline(MainForm->SP2bmhdc, mskpt, span - 1);

	if(MainForm->inSP2 && MainForm->mode == CW  &&
		   !(MainForm->Ldrag || MainForm->Hdrag))  // the moving line in SP2
	{
	   SelectObject(MainForm->SP2bmhdc, MainForm->greenpen);
	   MoveToEx(MainForm->SP2bmhdc, MainForm->mouseSP2.x, 0, NULL);
	   LineTo(MainForm->SP2bmhdc, MainForm->mouseSP2.x, MainForm->SP2H);
	}

	if(MainForm->CWPitch > 0. && MainForm->mode == CW)// the gray line in SP2
	{
	   MainForm->FF2yellowX = (MainForm->CWPitch - MainForm->FF2InizioScala)
												   / MainForm->FF2HzPerPixel;
	   SelectObject(MainForm->SP2bmhdc, MainForm->graypen);
	   MoveToEx(MainForm->SP2bmhdc, MainForm->FF2yellowX, 0, NULL);
	   LineTo(MainForm->SP2bmhdc, MainForm->FF2yellowX, MainForm->SP2H);
	}

// now compute the waterfall in WF2

	valquart = val[validx[sortBeg + sortSpan/4]]; // value of the quartile
	vrange = 160. - valquart;

	base = 10. * (float)MainForm->WF2Gain->Position / MainForm->WF2Gain->Max;

	if(MainForm->revWF1scroll)
	  pixptr = (Byte *)MainForm->WF2bm->ScanLine[MainForm->WF2H-1];
	else
	  pixptr = (Byte *)MainForm->WF2bm->ScanLine[0];

	if(!MainForm->WF2resizing && !MainForm->palettechanged)
	  for(int k=0; k<span; k++)
	  {
		  fm  = pow(max(0., base * (val[k] - valquart))/vrange, MainForm->wf2gamma);
		  idx = fm * 255. +0.5;
		  idx = min(idx, 255);
		  idx = max(1, idx);

		  pixptr[k] = (Byte)idx;
	  }

	if(MainForm->mode == CW)
	{
	  sprintf(st, "CW Pitch = %d Hz", (int)(MainForm->CWPitch + 0.5));
	  SelectObject(MainForm->SP2bmhdc, MainForm->CWfont);
	  SetTextColor(MainForm->SP2bmhdc, RGB(255, 255, 0));
	  TextOut(MainForm->SP2bmhdc, 380, 0, st, strlen(st));
	}

	EnterCriticalSection(&MainForm->screenUpdate);
	HDC hdcform = GetDC(MainForm->hwnd);
	BitBlt(hdcform, MainForm->SP2L, MainForm->SP2T,
					MainForm->SP2W, MainForm->SP2H,
					MainForm->SP2bmhdc, 0, 0, SRCCOPY);
	ReleaseDC(MainForm->hwnd, hdcform);

	LeaveCriticalSection(&MainForm->screenUpdate);

	return;
}
//---------------------------------------------------------------------------
void __fastcall TProcessFF2::ScrollWF2()
{
	if(MainForm->revWF1scroll)
	  MainForm->WF2bm->Canvas->CopyRect(MainForm->WF2src,
							   MainForm->WF2bm->Canvas, MainForm->WF2dest);
	else
	  MainForm->WF2bm->Canvas->CopyRect(MainForm->WF2dest,
							   MainForm->WF2bm->Canvas, MainForm->WF2src);
}
//---------------------------------------------------------------------------
void __fastcall TProcessFF2::PaintWF2()
{
	MainForm->Canvas->FrameRect(MainForm->FF2rect);
	MainForm->WF2->Repaint();
}
//---------------------------------------------------------------------------
int inline get2YfromDB(float dB)
{
	return MainForm->SP2rect.bottom  * (dB - MainForm->SP2toprange) /
					 (MainForm->SP2bottomrange - MainForm->SP2toprange) + 0.5;
}
//---------------------------------------------------------------------------
int sortfun(const void *a, const void *b)
{
	return (val[*(int *)a] < val[*(int *)b]) ? -1 : 1;
}
//---------------------------------------------------------------------------
int sortfun2(const void *a, const void *b)
{
	return (DynFacSpectr[*(int *)a] < DynFacSpectr[*(int *)b]) ? -1 : 1;
}
//---------------------------------------------------------------------------
void  __fastcall TMainForm::ComputeDynFactors(double *FF2audio)
{
	static int size = 0, order, direct, inverse;
	double HzPerBin;
	DCplx  FF2data[BUFSIZE + 2];
	int    j, bpBeg, bpEnd, bpSpan, ipeak;
	int    validx[BUFSIZE/2];
	float  f, fpeak, delta, quart, valquart, valpeak;

	if(size != BUFSIZE)
	{
	  size    = BUFSIZE;
	  order   = (int)(log(size)/log(2.) +0.0001);  // povero Borland....
	  direct  = NSP_Forw | NSP_NoBitRev;
	  inverse = NSP_Inv  | NSP_NoBitRev;

// compute the resapling indexes for the Dynamic Factor Indexes vector
	  resamp(size/2, AFFTLEN/2, MainForm->DynIdx);
	}

	HzPerBin = MainForm->vsrOut / (double)BUFSIZE;
	nspdRealFftNip(FF2audio, FF2data, order, direct);

	// compute magnitude : complex input; real output; double precision
	nspzbMag(FF2data, DynFacSpectr, size/2 + 1);

	for (j=0; j < size/2; j++) validx[j] = j;

	bpBeg = MainForm->bpLocut / HzPerBin + 0.5;
	bpEnd = MainForm->bpHicut / HzPerBin + 0.5;
	bpSpan = bpEnd - bpBeg;

	qsort(validx + bpBeg, bpSpan, sizeof(validx[0]), sortfun2);

	ipeak = validx[bpBeg + bpSpan - 1];// index in DynFacSpectr of the peak
	valquart = DynFacSpectr[validx[bpBeg + bpSpan/4]]; // value of the quartile
	valpeak  = DynFacSpectr[ipeak];                    // value of the peak
	delta    = max(0.1F, valpeak - valquart);

	for (j=0; j<BUFSIZE/2; j++)
	{
	   if(j < bpBeg || j >= bpBeg + bpSpan) MainForm->DynF[j] = 0.;
	   else
	   {
		 f = DynFacSpectr[j];
//         f = max(1.F, f/valquart);
//         MainForm->DynF[j] = 1. * log10(f);
		 f = max(0.01F, (f - valquart)) / delta;
		 f = pow(f, 1.2);
//         f = log10(1.01 + 8.99 * f);
		 MainForm->DynF[j] = 1.;
//         MainForm->DynF[j] = (4. * MainForm->DynF[j] + f) / 5.;
	   }
	}
}
//---------------------------------------------------------------------------
void  __fastcall TMainForm::WideBandDenoiser(float *FF2audio)
{
#define LMSLEN 32
#define DLYLENCW  2
#define DLYLENSSB 1
#define MUCW    1.e-5
#define MUSSB   5.e-5
#define LEAKCW  3.e-4
#define LEAKSSB 1.e-4

	static float pk = 0.;
	static NSPLmsState lmsSt;
	static float mu, lk, taps[LMSLEN + DLYLENCW], err, x, y;
	static int   dlyLen;
	int k;

begin:
	if(!MainForm->DenoiserInitialized)
	{
	   if(mode == CW)
	   { dlyLen = DLYLENCW;
		 mu     = MUCW;
		 lk     = LEAKCW;
	   }
	   else
	   { dlyLen = DLYLENSSB;
		 mu     = MUSSB;
		 lk     = LEAKSSB;
	   }

	   for(k=0; k < LMSLEN + dlyLen; k++) taps[k] = 0.;
	   err = 0.;
	   nspsLmsInit(NSP_LmsDefault, taps, LMSLEN, NULL, mu * MainForm->Den1fac,
				   lk, dlyLen, &lmsSt);
	   MainForm->DenoiserInitialized = true;
	}

	if(!MainForm->overload)
	   pk = nspsMax(FF2audio, BUFSIZE);
	else
	{
	   MainForm->DenoiserInitialized = false;
	   MainForm->overload = false;
	   goto begin;
	}

	nspsLmsSetLeak(pk * lk, &lmsSt);

	if(Den1AdjForm->Den1Changed)
	{
	  Den1AdjForm->Den1Changed = false;
	  nspsLmsSetStep(mu * MainForm->Den1fac, &lmsSt);
	}

	try
	{
	  for(k=0; k < BUFSIZE; k++)
	  {
		 x = FF2audio[k];
		 if(fabs(x) > 10.)  x = FF2audio[k] = 0.;
		 y = nspsLms(&lmsSt, x, err);
		 if(fabs(y) > 100.)      // some arbitrary, greater than 1., value....
		 {
		   MainForm->DenoiserInitialized = false;
		   FF2audio[k] = 0.;
		   goto begin;
		 }
		 err = x - y;
		 if(MainForm->nr) FF2audio[k] = y;
	  }
	}
	catch(...)
	{
	   MainForm->DenoiserInitialized = false;
	   goto begin;
	}
}
//---------------------------------------------------------------------------
float __fastcall ComputePhase(SCplx *vec)
{
	static double meanang = 0., begang = 0., mbegang = 0., iirfac = 1., dispang = 0.;
	double  Q0, I0, Q1, I1, ang0, ang1, deltang, err;
	static SCplx  Ctmp[BUFSIZE + 1];
	static int cnt = 0;

	if(MainForm->locked)
	  nspcbFir(&MainForm->PMLPstateNarr, vec, Ctmp + 1, BUFSIZE);
	else
	  nspcbFir(&MainForm->PMLPstateWide, vec, Ctmp + 1, BUFSIZE);

	for(int k = 0; k < BUFSIZE; k++)
	{
	   Q0 = Ctmp[k].im;   I0 = Ctmp[k].re;        Q1 = Ctmp[k+1].im;   I1 = Ctmp[k+1].re;

	   if(fabs(I0) < 1.e-13 || fabs(I1) < 1.e-13) continue;        // avoid discontinuites

	   ang0 = atan(fabs(Q0) / fabs(I0));
	   if(Q0 >= 0. && I0 < 0.)  ang0 = PI - ang0;     // 2nd quadrant
	   if(Q0 < 0.  && I0 < 0.)  ang0 = ang0 - PI;     // 3rd quadrant
	   if(Q0 < 0.  && I0 >= 0.) ang0 = -ang0;         // 4th quadrant

	   if(MainForm->mode == ECSS)
	   {
		 ang1 = atan(fabs(Q1) / fabs(I1));
		 if(Q1 >= 0. && I1 < 0.)  ang1 = PI - ang1;     // 2nd quadrant
		 if(Q1 < 0.  && I1 < 0.)  ang1 = ang1 - PI;     // 3rd quadrant
		 if(Q1 < 0.  && I1 >= 0.) ang1 = -ang1;         // 4th quadrant

		 deltang = ang1 - ang0;

// Cosi' si aggancia meglio....
		 if(deltang <= -PI/2) deltang = -PI/2;
		 else
		 if(deltang >= PI/2)  deltang = PI/2;

		 meanang = (meanang * 399. + deltang) / 400.;

		 mbegang = (mbegang * 19. + begang) / 20.;
		 err = meanang * 15. + mbegang/1000.;
		 if(abs(err) < 1.e-3) iirfac = 99.;   else iirfac = 1.;
		 MainForm->freqerr = (MainForm->freqerr * iirfac + err) / (iirfac + 1.);
		 if(fabs(MainForm->freqerr) < 8.e-2F) MainForm->locked = true;
		 else MainForm->locked = false;
	   }
	   else
	   {
		 MainForm->freqerr = 0.F;
		 MainForm->locked = false;
	   }
	}

	Ctmp[0] = Ctmp[BUFSIZE];
	begang = ang0;

	if(MainForm->mode == ECSS)
	  dispang = (dispang * 9. + ang0) / 10.;
	else
	  dispang = ang0;

	return dispang * 360. / twoPI;
}
//---------------------------------------------------------------------------


