//---------------------------------------------------------------------------
// This is the audio processign threads, which runs at a higher priority than the
// other threads in the program
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

// FIR coefficients (Sample rate: 192 KSps, Ft = 12 KHz, F0 = 19 KHz)
float fir_taps[] = {-0.00071855564601719379,
-0.0018417153041809797,
-0.0035491809248924255,
-0.0056731300428509712,
-0.0075738951563835144,
-0.0081593366339802742,
-0.0060697179287672043,
2.6207410319104254e-18,
0.010914741083979607,
0.02675328217446804,
0.046604625880718231,
0.068580500781536102,
0.090065531432628632,
0.10816453397274017,
0.12025231122970581,
0.12449999898672104,
0.12025231122970581,
0.10816453397274017,
0.090065531432628632,
0.068580500781536102,
0.046604625880718231,
0.02675328217446804,
0.010914741083979607,
2.6207410319104254e-18,
-0.0060697179287672043,
-0.0081593366339802742,
-0.0075738951563835144,
-0.0056731300428509712,
-0.0035491809248924255,
-0.0018417153041809797,
-0.00071855564601719379,
0.0,
0.0};

#define FIR_SIZE 32
#define STEREO_PILOT_THRESHOLD 1.

#include "ProcessIn.h"

//---------------------------------------------------------------------------
__fastcall TProcessIn::TProcessIn(bool CreateSuspended)
		: TThread(CreateSuspended)
{
	ResetEvent(MainForm->InBufferReady);
	ResetEvent(MainForm->InTerminate);
	offs = dstLen = hangcnt = 0;
	ZeroMemory(outLup, (BUFSIZE*MAXDOWN) * sizeof(int));
	ZeroMemory(outRup, (BUFSIZE*MAXDOWN) * sizeof(int));
	MainForm->PINidle = true;
	inIdx = 0;

	sleeptime = (float)BUFSIZE / MainForm->srateIn * 1000.;
}
//---------------------------------------------------------------------------
void __fastcall TProcessIn::Execute()
{
   DWORD whichevent, numbytes;
   HANDLE AudioEvents[2] = {MainForm->InTerminate, MainForm->InBufferReady};

   bool   found, outbReady;
   int    idxIn, idxOut, j, k;
   short  outL16[BUFSIZE], outR16[BUFSIZE];
   short  *out;
   static int inL32[BUFSIZE], inR32[BUFSIZE], outL32[BUFSIZE], outR32[BUFSIZE],
			  outL[BUFSIZE],   outR[BUFSIZE];

   while (!Terminated)
   {
	  whichevent = WaitForMultipleObjects(2, AudioEvents, FALSE, INFINITE);

      if(whichevent == WAIT_OBJECT_0)   // if audio terminate
	  {
		break;
	  }

// if not terminate, then must be audio buffer ready
	  if(whichevent == (WAIT_OBJECT_0 + 1))  // if audio ready
	  {
		MainForm->PINidle = false;
		if(MainForm->wavin)  // if reading from a WAV file
		{
		  if(!MainForm->paused)  // and not paused...
			 if(MainForm->driver == WMME)
				goto WMME_processing;
			 else
				goto ASIO_processing;   // should never happen...
		  else  // if paused clear the ASIO buffers to prevent noise
		  {
			if(MainForm->driver == ASIO)
			{
			  ZeroMemory(outL32, BUFSIZE * sizeof(int));
			  ZeroMemory(outR32, BUFSIZE * sizeof(int));
			  putASIOsamples(outL32, outR32);
			}
			continue; // reiterate while for next event (paused)
		  }
		}

		if(MainForm->extio)
		   MainForm->inQued = GetExtIOdata();

newExtIOdata:
// if WMME drivers
		if(MainForm->driver == WMME)
WMME_processing:
		{
		  while(MainForm->inQued > 0)
		  {
			if(!MainForm->wavin)
			{
			  if(!MainForm->extio && MainForm->inQued >= NINBUF - 2)
			  {
				 SoundCloseIn();   SoundCloseOut();
				 SoundOpenIn();  waveInStart(hin);  SoundOpenOut();
				 break;
			  }

			  if(!getBufPtrIn(idxIn, inL, inR))  // shift to 24 done inside the getBufPtrIn
				break;                           // unless data are floats32

			  ReQueIn(idxIn);

			  if(!sndOutOpen && !MainForm->extio)
				goto waitNextevent;  // reiterate while not terminated
			}

			MainForm->inQued--;

			if(!MainForm->wavin && MainForm->outQued > OUTBUFFERMARK + 10) //NOUTBUF/8 * 7)
			  goto skipThis;

// ----------- Process audio data here ----------------------
doit_again:
			outbReady = ProcessAudio(inL, inR, outL, outR);
// ----------------------------------------------------------

			if(outbReady)
			{
// check if upsampling needed, and, if yes, do it
			  dstLen = UpsampleMME(outL, outR, outLup, outRup);
			  offs = 0;

			  while(dstLen >= BUFSIZE)
			  {
				out = getBufPtrOut(idxOut); // idxOut is returned from the function
				if(out)
				{ for(j = 0, k = 0; k < BUFSIZE*2;)
				  {
					out[k++] = outLup[j   + offs] >> 8;
					out[k++] = outRup[j++ + offs] >> 8;
				  }
				  ReQueOut(idxOut);
				  offs += BUFSIZE;

				  if(MainForm->outQued <= 1)
				  {
					waveOutPause(hout);
					restarted = false;
				  }
				}
				else
				{ waveInStop(hin);
				  while(MainForm->outQued > NINBUF/4) // process some outbuffers
					 Sleep(64);
				  waveInStart(hin);
				  goto waitNextevent;                 // before continuing
				}
				dstLen -= BUFSIZE;
			  }
			  MainForm->normQue = MainForm->outQued;
			}   // if outbReady

			if(!MainForm->wavin && restarted && MainForm->outQued < OUTBUFFERMARK - 2)
			{
//			  goto doit_again;
			}
skipThis:
			if(MainForm->wavin && !MainForm->paused)
			{
recheck:
			  if(MainForm->outQued <= OUTBUFFERMARK)
			  {
wread:          MainForm->PIn->wreadlen = BUFSIZE;
				MainForm->IWav->InRead(inL, inR, MainForm->PIn->wreadlen);

				if(MainForm->PIn->wreadlen != BUFSIZE)
				{
				  if(MainForm->wlooping)
				  {
					MainForm->IWav->InRewind();
					goto wread;
				  }

				  ResetOut();
				  MainForm->paused = true;
				  MainForm->IWav->InRewind();
				  goto waitNextevent;  // reiterate while not terminated
				}

				MainForm->inQued++;
				SetEvent(MainForm->InBufferReady);  // this cause an immediate reiteration, but
				goto waitNextevent;                 // with a check first for termination
			  }
// if through here, outQued > OUTBUFFERMARK
			  Sleep(50);  // wait one buffer time (about...)
			  goto recheck;
			}   // if wavin

		  }    // while(MainForm->inQued > 0)

		  MainForm->PINidle = true;
		}
		else   // if not WMME, then ASIO
		{
ASIO_processing:
		  while(--MainForm->asioINfilled >= 0)
		  {
// get new audio data
			if(MainForm->wavin && MainForm->EoF)
			{
			  MainForm->WPauseClick(NULL);
			  MainForm->IWav->InRewind();
			  MainForm->EoF = false;
			  goto waitNextevent;
			}

			getASIOsamples(inL32, inR32);

			for(k=0; k < BUFSIZE; k++)
			{
			  inL[k]  = inL32[k] >> 8;
			  inR[k]  = inR32[k] >> 8;
			}

			if(!ProcessAudio(inL, inR, outL, outR))
			  continue;   // reiterate while asioInfilled >= 0

			dstLen = UpsampleASIO(outL, outR, outLup, outRup);
			offs = 0;

			while(dstLen >= BUFSIZE)
			{
			  for(k=0; k < BUFSIZE; k++)
			  {
				 outL32[k] = outLup[k + offs] << 8;
				 outR32[k] = outRup[k + offs] << 8;
			  }
			  offs += BUFSIZE;
// write audio data
			  putASIOsamples(outL32, outR32);
			  dstLen -= BUFSIZE;
			}

		  }
		  continue;   // reiterate while(!Terminated) loop
		} // end ASIO case

	  }  // if audio ready

	  if(MainForm->extio && (MainForm->inQued = GetExtIOdata()) > 0)
		 goto newExtIOdata;

waitNextevent:
   }   // while not terminated
}
//---------------------------------------------------------------------------
// processing in ProcessAudio is done with 24-bit resolution,
// Execute will shift WMME from 16 to 24 and ASIO from 32 to 24

bool __fastcall TProcessIn::ProcessAudio(void *inL24,  void *inR24,
										 int  *outL24, int  *outR24)
{
	int    j, k, len;
	SCplx  LRsamp[2 * BUFSIZE], LRsampP[BUFSIZE], LRvideo[BUFSIZE], *LRptr;
	float  Lsamp[BUFSIZE+4], Rsamp[BUFSIZE+4];   // the separate L and R samples (+4 for the delay)
	float  LsampAbs[BUFSIZE], RsampAbs[BUFSIZE]; // the absolute values
	float  Samp[BUFSIZE];                  // either L or R or both added
	SCplx  tmpSamp[3*BUFSIZE];               // temporary decimated samples
	SCplx  tmpSampQ[3*BUFSIZE];               // temporary decimated samples
	static int tmpSampQindex = 0;
	SCplx  downLRsamp[BUFSIZE];            // the decimated LR samples
	SCplx  buffLRsamp[BUFSIZE*2];          // the decimated LR samples
										   // accumulated in a BUFSIZE buffer
	SCplx  FF1fsamp[AFFTLEN + 2], FF1fsampECSS[AFFTLEN + 2];
	SCplx  IQtmp[AFFTLEN];                 // temporary for ComputePhase

	float FF2temp[AFFTLEN + 2], dtmp;
	float FF2tempStereo[AFFTLEN + 2];
	float FF2audio[BUFSIZE], FF2audioECSS[BUFSIZE];
	float FF2audioStereo[BUFSIZE];
	float mag[AFFTLEN], ph[AFFTLEN];
	float  tmp, d_downbuf[BUFSIZE], *pl, *pr;
	bool  over;

	static int loop2ctr = 0, direct, inverse, size=0, order;
	static SCplx AudioBuff[AFFTLEN], noiseMask[AFFTLEN];

	static float ss, pk, pk2, smlev = 0., denorm = 0., LoCutNorm, HiCutNorm,
				 AudioHzPerBin, peakvol, normconst;
	static int   tmpbuf[BUFSIZE], bufLen, bufIdx;
	static int   tmpbufStereo[BUFSIZE];
	// AlpAdd - B
	static SCplx ts0;
	static float rs0;
	float fir_acc, fir_acc_LR;
	static float fir_state[32];
	static float fir_state_LR[32];
	static int fir_idx = 0;
	static int fir_idx_LR = 0;
	static float xkm1, ykm1;
	float  tmpSampStereo[BUFSIZE];
	int wFMStereo = 0;
	static int LRsampindex = 0;
	// AlpAdd - E


// init FFT and other things
	if(size != AFFTLEN)
	{
	  nspzFreeTwdTbls();
	  nspFreeBitRevTbls();

	  size    = AFFTLEN;
	  order   = (int)(log(size)/log(2.) +0.0001); // povero Borland....
	  direct  = NSP_Forw | NSP_NoBitRev;
	  inverse = NSP_Inv  | NSP_NoBitRev;
	  nspzFft(NULL, order, (direct | NSP_Init));
	  for(k=0; k<AFFTLEN + 2; k++) FF1fsamp[k].re = FF1fsamp[k].im = 0.;
	}

	if(MainForm->justStarted)
	{
	  AudioHzPerBin = MainForm->vsrOut / size;
	  ss = 0.; pk = pk2 = 0.;  peakvol = 0.;
	  bufLen = bufIdx = 0;
	  ZeroMemory(AudioBuff, AFFTLEN * sizeof(SCplx));

	  if((MainForm->extio && MainForm->perseus)  ||   // if Perseus HW connected
		 (MainForm->wavin && MainForm->perseuswav))   // or Perseus wav playback

		 normconst = NORMCONSTP;
	  else
		 normconst = NORMCONST;

	  if(MainForm->extHWtype == SDR14)
		 normconst /= 3.1623F;         // attenuate SDR-14 & SDR-IQ by 10 dB
	}
// if needed, compute the new passband filter
	if(MainForm->bpchanged)
	{
	  LoCutNorm = MainForm->bpLocut/MainForm->vsrOut;
	  HiCutNorm = MainForm->bpHicut/MainForm->vsrOut;
	  ComputeMask(MainForm->FF2mask, size, order, LoCutNorm, HiCutNorm);
	  MainForm->Compute_IIR_parms();
	  MainForm->bpchanged = false;
	}

// Give out the raw data, if an entry point for this exists in an extio DLL

	if(MainForm->RawDataReady)
	  MainForm->RawDataReady(MainForm->srateIn, inL24, inR24, BUFSIZE);

	if(MainForm->wrec == wrecording  && !MainForm->werror)
	  if(!MainForm->OWav->OutWrite((unsigned int *)inL24, (unsigned int *)inR24, BUFSIZE))
		MainForm->werror = true;

	pl = Lsamp;  pr = Rsamp;  // point to the beginning of the buffers

	for(int k = 0; k < MainForm->rightdly; k++)
	{
	  *pr = Rsamp[BUFSIZE + k];  pr++;  // delay right channel as left
	}

	for(int k = 0; k < MainForm->leftdly; k++)
	{
	  *pl = Lsamp[BUFSIZE + k];  pl++;  // delay left channel as right
	}

	if(MainForm->extHWtype == USBfloat32)
	{
	  memcpy(pl, inL24, BUFSIZE * sizeof(float));
	  memcpy(pr, inR24, BUFSIZE * sizeof(float));
	}
	else
	{
	  nspsbIntToFloat(inL24, pl, BUFSIZE, 8*sizeof(int), NSP_Noflags);
	  nspsbIntToFloat(inR24, pr, BUFSIZE, 8*sizeof(int), NSP_Noflags);
    }

	if(MainForm->extio && MainForm->IQoffset != 0.F &&
						  (MainForm->extHWtype == SDR14 ||
						   MainForm->extHWtype == USBdata16 ||
						   MainForm->extHWtype == USBdata24))
	{
	  for(k = 0; k < BUFSIZE; k++)
	  {
		Lsamp[k] += MainForm->IQoffset;
		Rsamp[k] += MainForm->IQoffset;
	  }
	}

	if(MainForm->extHWtype != USBfloat32)  // if floats, they are already normalized
	{
	  nspsbMpy1(normconst, Lsamp, BUFSIZE);   // normalize to 1.
	  nspsbMpy1(normconst, Rsamp, BUFSIZE);   //  -----------
	}

	switch(MainForm->datamode)
	{
	  case mLeft      : nspcb2RealToCplx(Lsamp, Lsamp, LRsampP, BUFSIZE);
						break;

	  case mRight     : nspcb2RealToCplx(Rsamp, Rsamp, LRsampP, BUFSIZE);
						break;

	  case mLeftRight : nspsbAdd3(Lsamp, Rsamp, Samp, BUFSIZE);
						nspcb2RealToCplx(Samp, Samp, LRsampP, BUFSIZE);
						break;

	  case mIQ        : if(MainForm->skew  != 0.)
//                           FDFilter(Lsamp, Rsamp);
						{
						  memcpy(Samp, Rsamp, sizeof(float)*BUFSIZE);
						  nspsbMpy1(MainForm->skew, Samp, BUFSIZE);
						  nspsbAdd2(Samp, Lsamp, BUFSIZE);
						}
						if(MainForm->Qmult != 1.)
						   nspsbMpy1(MainForm->Qmult, Rsamp, BUFSIZE);
						if(MainForm->swapIQ)
						   nspcb2RealToCplx(Rsamp, Lsamp, LRsampP, BUFSIZE);
						else
						   nspcb2RealToCplx(Lsamp, Rsamp, LRsampP, BUFSIZE);
						break;
	}

// Pre FF1 processing

//	memcpy(LRvideo, LRsampP, sizeof(SCplx) * BUFSIZE);

	if(MainForm->nblanker)
	{

	   static float avgmag = 0, avglvlRe = 0., avglvlIm = 0., mag;
	   static SCplx avglvl = SCplxZero;
	   SCplx lvl;

	   for(k = 0; k < BUFSIZE; k++)
	   {
		 int j;

		 mag = sqrt(LRsampP[k].re * LRsampP[k].re + LRsampP[k].im * LRsampP[k].im);
		 avgmag = 0.99999 * avgmag + 0.00001 * mag;
		 avglvlRe = 0.999 * avglvlRe + 0.001 * LRsampP[k].re;
		 avglvlIm = 0.999 * avglvlIm + 0.001 * LRsampP[k].im;

		 if(mag > (MainForm->BlkThr * avgmag * 0.55))
		 {
		   LRsamp[k].re = (LRsampP[k].re > 0.) ? avglvlRe : -avglvlRe;
		   LRsamp[k].im = (LRsampP[k].im > 0.) ? avglvlIm : -avglvlIm;

		   for(j = 1; j < 3 && (k - j) >= 0; j++)
		   {  LRsamp[k-j].re = (LRsampP[k-j].re > 0.) ? avglvlRe : -avglvlRe;
			  LRsamp[k-j].im = (LRsampP[k-j].im > 0.) ? avglvlIm : -avglvlIm;
		   }

		   for(j = 1; j < 3 && (k + j) < BUFSIZE; j++)
		   {  LRsamp[k+j].re = (LRsampP[k+j].re > 0.) ? avglvlRe : -avglvlRe;
			  LRsamp[k+j].im = (LRsampP[k+j].im > 0.) ? avglvlIm : -avglvlIm;
		   }

		 }
		 else  // if threshold not exceeded
		   LRsamp[k] = LRsampP[k];

	   }
	   LRptr = LRsamp;  // point to "blanked" buffer
	}
	else  // if no blanker activated
	  LRptr = LRsampP;  // point to original (unblanked) buffer

	EnterCriticalSection(&MainForm->acFF1que);
	{
	  memcpy(MainForm->PFF1->PolyQue, MainForm->PFF1->PolyQue + BUFSIZE,
			 (MainForm->ff1Size * MainForm->polyfactor - BUFSIZE) * sizeof(SCplx));

	  memcpy(MainForm->PFF1->PolyQue + (MainForm->polyfactor * MainForm->ff1Size - BUFSIZE),
			 LRptr, BUFSIZE * sizeof(SCplx));
	}
	LeaveCriticalSection(&MainForm->acFF1que);

//*************************************
	if(MainForm->despreading)
	   PreFF1Proc(LRptr);
//*************************************

// downconvert the complex (L, R) input
	DownConvert(LRptr, tmpSamp);


	//
	// AlpAdd - BEGIN
	//

	// For wide FM we must perform frequency
	// demodulation before to downsampling, since
	// downsampling must be carried out in
	// time domain.

	// In this case, switch back to narrowband FM
	if(MainForm->mode == FM && MainForm->submode != lower &&
		MainForm->srateIn < 96000)
	{
		MainForm->submode = lower;
		MainForm->AdjustBlueWindow();
	}

	double Fs = MainForm->srateIn; 		// Sample rate [samples per second]
	double Fa = MainForm->srateOut;
	double f_t;
	int order1;
	double gfact;
	if(MainForm->mode == FM && MainForm->submode != lower)
	{





		// Low-pass filter
		f_t = MIN(125.e3, MainForm->srateIn / 2.);
		order1 = int(log(BUFSIZE + 1) / log(2.));
		nspcFft(tmpSamp, order1, direct);
		for(k = BUFSIZE * f_t / Fs;
			k < (BUFSIZE * (Fs - f_t))/Fs;
			k++)
				tmpSamp[k].re = tmpSamp[k].im = 0;

		nspcFft(tmpSamp, order1, inverse);
		gfact = Fs / (2. * f_t);
		for(k = 0; k < BUFSIZE; k++)
		{
			tmpSamp[k].re *= gfact;
			tmpSamp[k].im *= gfact;
		}






	  // Demodulate
	  float divisor = 0.;
	  for( k=0; k<BUFSIZE; k++ )
	  {
		 divisor = sqrt(tmpSamp[k].re * tmpSamp[k].re +
				   tmpSamp[k].im * tmpSamp[k].im);
		 divisor += sqrt(ts0.re * ts0.re +
				   ts0.im * ts0.im);

		 if(divisor)
		 {
			rs0 = (tmpSamp[k].im * ts0.re -
				   tmpSamp[k].re * ts0.im) / divisor;
			ts0 = tmpSamp[k];
			tmpSamp[k].re = rs0;
			tmpSamp[k].re *= 1.e-2F
				* MainForm->srateIn / MainForm->srateOut;
		 }
		 else
		 {
			tmpSamp[k].re = 0.;
		 }
		 tmpSamp[k].im = 0;
	  }





	  //
	  // FM deemphasis filter
	  //
	  double tau = 75.e-6;      // RC constant of deemphasis filter [s]

	  double w_p = 1 / tau;
	  double w_pp = tan(w_p / 2 / Fs);

	  // float a0 = 1.;
	  float a1 = (w_pp - 1.) / (w_pp + 1.);
	  float b0 = w_pp / (w_pp + 1.);
	  float b1 = b0;
	  float tmpout;

	  for(k = 0; k < BUFSIZE; k++)
	  {
			tmpout = tmpSamp[k].re * b0;
			tmpout += xkm1 * b1;
			tmpout += ykm1 * a1;

			xkm1 = tmpSamp[k].re;
			ykm1 = tmpout;

			tmpSamp[k].re = tmpout;
	  }


	  //
	  // Stereo processing
	  //
	  double fp = 19000.;		// Frequency of pilot [Hz]
	  double Fnorm = fp / Fs;
	  double phi;
	  SCplx acc, pilot;
	  if(MainForm->submode == both && Fs >= 250000)
	  {

			// Detect the stereo pilot
			acc.re = acc.im = 0.;
			for(k = 0; k < BUFSIZE; k++)
			{
				pilot.re = sin(twoPI * Fnorm * k);
				pilot.im = cos(twoPI * Fnorm * k);
				acc.re += tmpSamp[k].re * pilot.re;
				acc.im += tmpSamp[k].re * pilot.im;
			}


			//
			// Check if the stereo signal is present
			//
			if(acc.re * acc.re + acc.im * acc.im > STEREO_PILOT_THRESHOLD)
			{
				// Enable wFM stereo mode
				wFMStereo = 1;

				// Phase of the pilot subcarrier
				phi = 2. * atan2(acc.im, acc.re);

				// Down-convert the L-R component and low-pass it
				for(k = 0; k < BUFSIZE; k++)
				{
					fir_state_LR[fir_idx_LR] = tmpSampStereo[k]
							* sin(twoPI * 2. * Fnorm * k + phi);

					fir_acc_LR = 0.;
					for(int l = 0; l < FIR_SIZE; l++)
					{
						fir_acc_LR +=
							fir_state_LR[(fir_idx - l + FIR_SIZE) % FIR_SIZE]
							* fir_taps[l];
					}
					tmpSampStereo[k] = fir_acc_LR;
					fir_idx_LR++;
					fir_idx_LR %= FIR_SIZE;
				}

				// Now tmpSampStereo holds the L-R signal;
			}
			else
			{
				// Disable wFM stereo mode
				wFMStereo = 1;
				
				for(k = 0; k < BUFSIZE; k++)
				{
					tmpSampStereo[k] = 0.;
				}
			}
			// Now tmpSampStereo holds the L-R signal or nothing
			// if the stereo pilot was not detected


	  }





	  //
	  // Low-pass filtering (ft = 15 kHz or less)
	  //
	  // Low-pass filter
	  f_t = MIN(15.e3, Fa / 2.);
	  if(Fs < 250.e3) f_t = MIN(12.e3, Fa / 2.);
	  if(Fs < 192.e3) f_t = MIN(6.e3, Fa / 2.);
	  order1 = int(log(BUFSIZE + 1) / log(2.));
	  nspcFft(tmpSamp, order1, direct);
	  for(k = BUFSIZE * f_t / Fs;
			k < (BUFSIZE * (Fs - f_t))/Fs;
			k++)
				tmpSamp[k].re = tmpSamp[k].im = 0;
	  nspcFft(tmpSamp, order1, inverse);
	  gfact = Fs / (2. * f_t);
	  for(k = 0; k < BUFSIZE; k++)
	  {
		tmpSamp[k].re *= gfact;
		tmpSamp[k].im *= gfact;
	  }

	}
	// Now we have a narrow-band audio signal (mono) in tmpSamp[].re

	//
	// My decimation - AlP - BEGIN
	//
	int order2;
	f_t = Fa / 2.;
	order1 = int(log(BUFSIZE + 1) / log(2.));
	order2 = ceil(order1 + log(Fa / double(Fs)) / log(2.));
	float ovl = 1. - (Fs * (1 << order2) / double(Fa * (1 << order1)));
    ovl = 1. - (BUFSIZE * Fa) / double(Fs * (1 << order2)); 
	int sstep =  BUFSIZE * (1. - ovl);

	// To frequency domain
	memcpy(tmpSampQ + tmpSampQindex, tmpSamp, BUFSIZE * sizeof(SCplx));
	tmpSampQindex += BUFSIZE;
	while(tmpSampQindex >= BUFSIZE)
	{
		memcpy(tmpSamp, tmpSampQ, BUFSIZE * sizeof(SCplx));
		memmove(tmpSampQ, tmpSampQ + sstep,
					(BUFSIZE * (1. + ovl)) * sizeof(SCplx));
		tmpSampQindex -= sstep;

		nspcFft(tmpSamp, order1, direct);

		// Normalize
		int Nfft2 = 1 << order2;
		gfact = Fs / Fa;
		for(k = 0; k < Nfft2 / 2; k++)
		{
			tmpSamp[k].re *= gfact;
			tmpSamp[k].im *= gfact;
		}
		for(; k < Nfft2; k++)
		{
			tmpSamp[k].re = gfact * tmpSamp[BUFSIZE - Nfft2 + k].re;
			tmpSamp[k].im = gfact * tmpSamp[BUFSIZE - Nfft2 + k].im;
		}

		// Back to time domain
		// LRsampindex %= BUFSIZE;
		nspcFftNip(tmpSamp, LRsamp + LRsampindex, order2, inverse);
		LRsampindex += Nfft2;

	}

	MainForm->justStarted = false;
	if(LRsampindex < BUFSIZE) return false;


	//
	// My decimation - AlP - END
	//

	//
	// AlpAdd - END
	//



// decimate the baseband signal, return false if a BUFSIZE buffer not ready yet
	/* if(!SincDecimate(tmpSamp, LRsamp))
	{
	  MainForm->justStarted = false;
	  return false;
	}
	MainForm->justStarted = false;
	*/
	
// queue the downconverted and decimated signal for the audio FFT
	memmove(AudioBuff, AudioBuff + BUFSIZE,  (AFFTLEN-BUFSIZE) * sizeof(SCplx));
	memmove(AudioBuff + (AFFTLEN-BUFSIZE), LRsamp, BUFSIZE * sizeof(SCplx));



// recover the phase offset between the AM carrier and the local NCO
	if(MainForm->mode == AM || MainForm->mode == ECSS)
		phase = ComputePhase(LRsamp);
	else
		MainForm->freqerr = phase = 0.F;

	Synchronize(&UpdatePM);

	memmove(LRsamp, LRsamp + BUFSIZE, BUFSIZE * sizeof(SCplx));    // AlpAdd
	LRsampindex -= BUFSIZE;             // AlpAdd

// compute the complex FFT of the downconverted audio
	nspcFftNip(AudioBuff, FF1fsamp, order, direct);
	if(MainForm->mode != AM && MainForm->mode != FM)
	   FF1fsamp[0].re = FF1fsamp[0].im = FF1fsamp[AFFTLEN-1].re = FF1fsamp[AFFTLEN-1].im =
	   FF1fsamp[1].re = FF1fsamp[1].im = FF1fsamp[AFFTLEN-2].re = FF1fsamp[AFFTLEN-2].im = 0.F;

// mirror the upper half of the FFT into the
// lower half, when processing the LSB
	if(MainForm->mode == LSB)
	  for(k=1, j=AFFTLEN-1; k<AFFTLEN/2; k++, j--)
	  {
		FF1fsamp[k].re =  FF1fsamp[j].re;
		FF1fsamp[k].im = -FF1fsamp[j].im;
	  }

	if(MainForm->mode == ECSS && MainForm->submode != upper)
	{
	  ZeroMemory(FF1fsampECSS, sizeof(SCplx) * (AFFTLEN +2));
	  for(k=1, j=AFFTLEN-1; k<AFFTLEN/2; k++, j--)
	  {
		FF1fsampECSS[k].re =  FF1fsamp[j].re;
		FF1fsampECSS[k].im = -FF1fsamp[j].im;
	  }
	}

	if(MainForm->fullview && MainForm->mode == CW)
	{
// do an inverse FFT of only the first half of the complex spectrum.
// recovering a real audio signal
	  nspsCcsFftNip(FF1fsamp, FF2temp, order, inverse);
// apply the overlap-and-save algorithm
	  memcpy(FF2audio, FF2temp + (AFFTLEN-BUFSIZE), BUFSIZE * sizeof(float));


	  EnterCriticalSection(&MainForm->acFF2que);
		 memmove(MainForm->PFF2->PolyQue, MainForm->PFF2->PolyQue + BUFSIZE,
				(MainForm->ff2Size * MainForm->polyfactor - BUFSIZE) * sizeof(float));
		 memcpy(MainForm->PFF2->PolyQue + (MainForm->polyfactor * MainForm->ff2Size - BUFSIZE),
				FF2audio, BUFSIZE * sizeof(float));
	  LeaveCriticalSection(&MainForm->acFF2que);
	}


// multiply the complex spectrum by the complex bandpass filter
	if(MainForm->mode != FM) nspcbMpy2(MainForm->FF2mask, FF1fsamp, AFFTLEN);

	if(MainForm->mode == ECSS && MainForm->submode != upper)
	  nspcbMpy2(MainForm->FF2mask, FF1fsampECSS, AFFTLEN/2);

	if(MainForm->mode == LSB || MainForm->mode == USB ||
	   MainForm->mode == CW || MainForm->mode == DRM)
	{
// do an inverse FFT of only the first half of the complex spectrum.
// recovering a real audio signal
	  nspsCcsFftNip(FF1fsamp, FF2temp, order, inverse);
// apply the overlap-and-save algorithm
	  memcpy(FF2audio, FF2temp + (AFFTLEN-BUFSIZE), BUFSIZE * sizeof(float));
	}
	else  if(MainForm->mode == ECSS)
	{
	  if(!MainForm->locked) goto am_demod;

	  if(MainForm->submode != lower)
	  {
// do an inverse FFT of only the first half of the complex spectrum.
// recovering a real audio signal from the upper spectrum (USB)
		nspsCcsFftNip(FF1fsamp, FF2temp, order, inverse);
// apply the overlap-and-save algorithm
		memcpy(FF2audio, FF2temp + (AFFTLEN-BUFSIZE), BUFSIZE * sizeof(float));
	  }

	  if(MainForm->submode != upper)
	  {
// do an inverse FFT of only the first half of the complex spectrum.
// recovering a real audio signal from the lower spectrum (LSB)
		nspsCcsFftNip(FF1fsampECSS, FF2temp, order, inverse);
// apply the overlap-and-save algorithm
		memcpy(FF2audioECSS, FF2temp + (AFFTLEN-BUFSIZE), BUFSIZE * sizeof(float));
	  }

	  if(MainForm->submode == both)
		nspsbAdd2(FF2audioECSS, FF2audio, BUFSIZE);

	  if(MainForm->submode == lower)
		memcpy(FF2audio, FF2audioECSS,  BUFSIZE * sizeof(float));

// if submode == upper, the vector FF2audio has already the right contents

	}
	else if(MainForm->mode == AM)
	{
am_demod:
	  nspcFft(FF1fsamp, order, inverse);           // inverse complex FFT
	  memcpy(tmpSamp, FF1fsamp + (AFFTLEN-BUFSIZE), sizeof(SCplx)*BUFSIZE);

	  nspcbMag(tmpSamp, FF2temp, BUFSIZE);         // just compute the magnitude and the
												   // demodulation is done !

	  HighPassFilter(FF2temp, FF2audio, BUFSIZE);  // cut everything below 150 Hz
	}
	else if(MainForm->mode == FM)
	{
//  FM demodulator ---------------------------------
	  float divisor;
	  const int len = (AFFTLEN - BUFSIZE)/2;

	  nspcFft(FF1fsamp, order, inverse);           // inverse complex FFT
	  memcpy(tmpSamp, FF1fsamp + (AFFTLEN-BUFSIZE), sizeof(SCplx)*BUFSIZE);

	if(MainForm->submode != lower) // AlpAdd:
	{                                     // AlpAdd:
										  // AlpAdd:
	  for( k=0; k<BUFSIZE; k++ )           // AlpAdd:
	  {                                   // AlpAdd:
		 FF2temp[k] = FF1fsamp[k].re;      // AlpAdd: just copy!
	  }                                    // AlpAdd:
	  if(MainForm->submode == both)             // AlpAdd
	  {                                        // AlpAdd
			HighPassFilterStereo(FF2tempStereo,   // AlpAdd
							FF2audioStereo,    // AlpAdd
							BUFSIZE);  // AlpAdd
	  }                                   // AlpAdd
	}                                     // AlpAdd:
	else                                 // AlpAdd:
	{                                    // AlpAdd:
	  for( k=0; k<BUFSIZE; k++ )
	  {

		 divisor = FF1fsamp[k+len].re * FF1fsamp[k+len].re +
				   FF1fsamp[k+len].im * FF1fsamp[k+len].im;

		 if( divisor != 0.0 )
		 {
			FF2temp[k] = (FF1fsamp[k+len].im * FF1fsamp[k+len-1].re -
						  FF1fsamp[k+len].re * FF1fsamp[k+len-1].im) / divisor;

			FF2temp[k] *= 5.e-3F;
		 }
		 else			// AlpAdd
		 {              // AlpAdd
			FF2temp[k] = k ? FF2temp[k - 1] : 0.;   // AlpAdd
		 }              // AlpAdd
	  }
	}                                     // AlpAdd:



	
	  HighPassFilter(FF2temp, FF2audio, BUFSIZE);  // cut everything below 150 Hz
	}

	if(!(MainForm->fullview && MainForm->mode == CW))
	{
	  EnterCriticalSection(&MainForm->acFF2que);
		 memmove(MainForm->PFF2->PolyQue, MainForm->PFF2->PolyQue + BUFSIZE,
			 (MainForm->ff2Size * MainForm->polyfactor - BUFSIZE) * sizeof(float));
		 memcpy(MainForm->PFF2->PolyQue + (MainForm->polyfactor * MainForm->ff2Size - BUFSIZE),
				FF2audio, BUFSIZE * sizeof(float));
	  LeaveCriticalSection(&MainForm->acFF2que);
	}

	if((loop2ctr++ % (21 - MainForm->WF2Speed->Position)) == 0)
	  SetEvent(MainForm->FF2BufferReady);  // compute a new FF2 screen, by
										   // awakening ProcessFF2, which then
										   // will call ComputeFF2

	if(MainForm->cwp)
	{ if(MainForm->mode == CW)
	  {
		MainForm->DoPeak(FF2audio);
		MainForm->DoPeak2(FF2audio);
	  }
	  else // USB or LSB
		;
	}

// ------------------ AGC ------------------------------
	for (k = 0; k < BUFSIZE; k++)
	{
		tmp = fabs(FF2audio[k]);

		if(ss < tmp)
		{ ss = (9. * ss + tmp) / 10.;    // slow down attack time
		  pk = tmp;
		  hangcnt = (MainForm->agc == fast) ? 1 : 6;
		}

		if(MainForm->agc != off)
		{						// AlpAdd
		  FF2audio[k] *= 0.9 / max(ss, MainForm->agcthr);
		  FF2audioStereo[k] *= 0.9 / max(ss, MainForm->agcthr);  // AlpAdd
		}	// AlpAdd
		else
		{		// AlpAdd
		  FF2audio[k] *= 3.e3 * MainForm->agcgain;
		  FF2audioStereo[k] *= 3.e3 * MainForm->agcgain;     //AlpAdd
		}       // AlpAdd
		if(hangcnt == 0)
		  ss  *= MainForm->agc_decay;

		pk  *= MainForm->agc_decay;
	}

	if(hangcnt > 0)  hangcnt--;
// ------------------------------------------------------

	MainForm->pklevel = pk;
	Synchronize(&UpdateVU);

	nspsbMpy1(ONEFIFTH, FF2audio , BUFSIZE); // denormalize...
	
	if(MainForm->mode == FM && MainForm->submode == both)  // AlpAdd
		nspsbMpy1(ONEFIFTH, FF2audioStereo, BUFSIZE); // AlpAdd: denormalize...

//	if(MainForm->nr)   // the rotine is always executed to allow for fast convergence
	{                  // but its output is used only when nr is true
	   MainForm->WideBandDenoiser(FF2audio);
	}


	float vol = pow((float)MainForm->vol/99., 2);
	if(MainForm->audiomute)
	  vol = 0.;

	denorm = vol * 3. * twoTO23m1 * (MainForm->nr ? 1.5 : 1.);
	denorm /= ONEFIFTH;

	nspsbMpy1(denorm, FF2audio , BUFSIZE); // denormalize...
	if(MainForm->mode == FM && MainForm->submode == both)  // AlpAdd
		nspsbMpy1(denorm, FF2audioStereo, BUFSIZE); // AlpAdd: denormalize...

	do
	{
	  nspsbAbs2(FF2audio, FF2temp, BUFSIZE);
	  peakvol = nspsMax(FF2temp, BUFSIZE);
	  if(peakvol > 0.995 * twoTO23m1)
	  {
		nspsbMpy1(0.98F, FF2audio , BUFSIZE);
		if(MainForm->mode == FM && MainForm->submode == both)  // AlpAdd
			nspsbMpy1(0.98F, FF2audioStereo, BUFSIZE); // AlpAdd: denormalize...
		over = true;
	  }
	  else over = false;

	} while(over);

	MainForm->overload = false;

// reconvert to integers
	nspsbFloatToInt(FF2audio, tmpbuf, BUFSIZE, 8*sizeof(int),
											   NSP_Round | NSP_Clip);


	if(MainForm->mode == FM && MainForm->submode == both)  // AlpAdd
	{                                                    // AlpAdd
		nspsbFloatToInt(FF2audioStereo, tmpbufStereo, BUFSIZE,      // AlpAdd
			8*sizeof(int), NSP_Round | NSP_Clip);        // AlpAdd

		for(int k=0, j=0; k<BUFSIZE; k++)                // AlpAdd
		{                                                // AlpAdd
		  outL24[j]   = tmpbuf[k] + tmpbufStereo[k];  // AlpAdd: left channel
		  outR24[j++] = tmpbuf[k] - tmpbufStereo[k];  // AlpAdd: right channel
		}                                             // AlpAdd
	}                                                 // AlpAdd
	else                                              // AlpAdd
	{                                                 // AlpAdd
		for(int k=0, j=0; k<BUFSIZE; k++)
		{
		  outL24[j]   = tmpbuf[k];  // left channel
		  outR24[j++] = tmpbuf[k];  // right channel
		}
	}                                                   // AlpAdd

	return true;
}
//---------------------------------------------------------------------------
void __fastcall TProcessIn::UpdateVU()
{
#define S9p50  0.1088    // measured pklevel at -23 dBm == S9 + 50 dB (S-meter full scale)
#define S1m12  0.344e-6  // pklevel at S1 - 12 dB (S-meter zero scale)

// 14.14 is 20 * 0.707 (RMS / peak)

	float val = 14.14F * log10(1.e-15 +  // this term only to prevent log(0)
						MainForm->pklevel / S9p50);

	MainForm->VU->Position = MainForm->VU->Scale + val;
}
//---------------------------------------------------------------------------
void __fastcall TProcessIn::UpdatePM()
{
	MainForm->PM->Value = int(phase);
}
//---------------------------------------------------------------------------
void __fastcall ComputeMask(SCplx *mask, int size, int order, float lo, float hi)
{
	double hre[4096];
	SCplx  hs[4096];
	int    k;

	lo = max(lo, 0.001F);    hi = max(0.002F, min(hi, 0.499F));
	hi = max(hi, lo + 0.001F);

	if(MainForm->mode == AM)
	  nspdFirLowpass(hi, hre, size - BUFSIZE + 1, NSP_WinBlackmanOpt, 1);
	else
	  nspdFirBandpass(lo, hi, hre, size - BUFSIZE + 1, NSP_WinBlackmanOpt, 1);
	for(k = size - BUFSIZE + 1; k < size; k++) hre[k] = 0.;
	for(k = 0; k < size; k++) {hs[k].re = hre[k]; hs[k].im = hs[k].re;}
	nspcFftNip(hs, mask, order, NSP_Forw | NSP_NoBitRev);
}
//---------------------------------------------------------------------------
int __fastcall UpsampleMME(int* outL, int* outR, int* outLup, int* outRup)
{
   int    dstLen;

   {
	 dstLen = BUFSIZE;
	 nspsbCopy((float *)outL, (float *)outLup, BUFSIZE);
	 nspsbCopy((float *)outR, (float *)outRup, BUFSIZE);
	 return dstLen;
   }
}
//---------------------------------------------------------------------------
int __fastcall UpsampleASIO(int* outL, int* outR, int* outLup, int* outRup)
{
   int    dstLen;
   static float oldcutoff = 0., cutoff;
   double foutLup[BUFSIZE * MAXDOWN], foutL[BUFSIZE];

   if(MainForm->downFactor == 1)
   {
	 dstLen = BUFSIZE;
	 nspsbCopy((float *)outL, (float *)outLup, BUFSIZE);
	 nspsbCopy((float *)outR, (float *)outRup, BUFSIZE);
	 return dstLen;
   }

   cutoff = (MainForm->vsrOut * 0.49) / MainForm->srateOut;

   if(oldcutoff != cutoff) // then compute the new anti-alias filter taps
   {
	 if(NSP_StsOk != nspdFirLowpass(cutoff, dtapValsUP, LPFIRUPLEN,
												   NSP_WinBlackmanOpt, 1))
	 { fatal("An error occurred when computing the upsampling anti-alias filter");
	   Application->Terminate();
	 }

	 if(oldcutoff != 0.)  // if at least one time through before...
	   nspFirFree(&MainForm->statePtrUP);

	 oldcutoff = cutoff;
	 nspdFirInitMr(dtapValsUP, LPFIRUPLEN, NULL, MainForm->downFactor, 0,
												  1, 0, &MainForm->statePtrUP);
   }

// and now finally upsample and lowpass the audio data
   dstLen = BUFSIZE * MainForm->downFactor;

// convert to doubles for upsampling
   nspdbIntToFloat(outL, foutL, BUFSIZE, 8*sizeof(int), NSP_Noflags);

// keep the volume constant, despite the upsampling
   nspdbMpy1(MainForm->downFactor, foutL, BUFSIZE);

// do the upsampling followed by filtering
   nspdbFir(&MainForm->statePtrUP,  foutL, foutLup, BUFSIZE);

// reconvert to integers
   nspdbFloatToInt(foutLup, outLup, dstLen, 8*sizeof(int),
											   NSP_Round | NSP_Clip);
// for the time being, the right channel is equal to the left channel
   nspsbCopy((float *)outLup, (float *)outRup, dstLen);

   return dstLen;
}//---------------------------------------------------------------------------
int __fastcall Median5(float *q, int k)
{
	int idx[5] = {0, 1, 2, 3, 4};
	float p[5];

#define PIX_SORT(a,b) { if (p[a]>p[b]) PIX_SWAP(a,b); }
#define PIX_SWAP(a,b) { float temp=p[a]; p[a]=p[b]; p[b]=temp; \
						int t=idx[a]; idx[a]=idx[b]; idx[b]=t; }

	memcpy(p, q + k - 2, 5 * sizeof(float));
	PIX_SORT(0,1) ; PIX_SORT(3,4) ; PIX_SORT(0,3) ;
	PIX_SORT(1,4) ; PIX_SORT(1,2) ; PIX_SORT(2,3) ;
	PIX_SORT(1,2) ;
	return k - 2 + idx[2];
}
//---------------------------------------------------------------------------
// swaps the first half of the vector with the second half
void __fastcall Swap(DCplx vec[], int size)
{
   int k, j; DCplx tmp;

   for(k=0; k<size/2; k++)
   {
	 j = k + size/2;

	 tmp    = vec[k];
	 vec[k] = vec[j];
	 vec[j] = tmp;
   }
}
//---------------------------------------------------------------------------
