//---------------------------------------------------------------------------
// This routine manage the ASIO callbacks, to not be confused with the ExtIO callbacks
//---------------------------------------------------------------------------
#include <vcl.h>
#include "pch.h"
#pragma  hdrstop

#include "callbacks.h"

static int bufindex, len, tL32[RINGSIZE], tR32[RINGSIZE];

//---------------------------------------------------------------------------
void bufferSwitch(long index, ASIOBool directProcess)
{

// the actual processing callback.
// Beware that this is normally in a seperate thread, hence be sure that
// you take care about thread synchronization. This is omitted here for
// simplicity.

// as this is a "back door" into the bufferSwitchTimeInfo a timeInfo needs
// to be created though it will only set the timeInfo.samplePosition and
// timeInfo.systemTime fields and the according flags

	ASIOTime  timeInfo;
	memset( &timeInfo, 0, sizeof (timeInfo) );

// Call the real callback
	  bufferSwitchTimeInfo( &timeInfo, index, directProcess );
}
//---------------------------------------------------------------------------
void sampleRateChanged(ASIOSampleRate sRate)
{
	(void) sRate; /* unused parameter */
}
//---------------------------------------------------------------------------
long asioMessages(long selector, long value, void* message, double* opt)
{
// currently the parameters "message" and "opt" are not used.
	long ret = 0L;

	(void) message; /* unused parameters */
	(void) opt;

	switch(selector)
	{
		case kAsioSelectorSupported:
			if(value == kAsioResetRequest
			|| value == kAsioBufferSizeChange
			|| value == kAsioEngineVersion
			|| value == kAsioResyncRequest
			|| value == kAsioLatenciesChanged
			|| value == kAsioSupportsTimeInfo
			|| value == kAsioSupportsTimeCode)
			   ret = 1L;
			else
			   ret = 0L;

			break;

		case kAsioBufferSizeChange:
			ret = 1L;
			break;

		case kAsioResetRequest:
			ret = 1L;
			break;

		case kAsioResyncRequest:
			ret = 1L;
			break;

		case kAsioLatenciesChanged:
			ret = 1L;
			break;

		case kAsioEngineVersion:
			ret = 2L;
			break;

		case kAsioSupportsTimeInfo:
			ret = 1;
			break;

		case kAsioSupportsTimeCode:
			ret = 0;
			break;
	}
	return ret;
}
//---------------------------------------------------------------------------
ASIOTime *bufferSwitchTimeInfo( ASIOTime *timeInfo, long index, ASIOBool directProcess )
{
	void *pL, *pR;

	bufindex = index;
	len = MF->asioDriverInfo.preferredSize; // current ASIO buffer size set by the user

	if(MainForm->wavin && MainForm->paused) // currently, wavin is managed only with WMME....
	{
	  ZeroMemory(MF->asioDriverInfo.bufferInfos[MF->LChO].buffers[bufindex], len * MF->asioSampleLength);
	  ZeroMemory(MF->asioDriverInfo.bufferInfos[MF->RChO].buffers[bufindex], len * MF->asioSampleLength);
	  if(MF->asioDriverInfo.postOutput) ASIOOutputReady();
	  return 0L;
	}

// from ring to card ------------------------------------------

	if(MF->OLring->getlen() < len)
	{
	  ZeroMemory(MF->asioDriverInfo.bufferInfos[MF->LChO].buffers[bufindex], len * MF->asioSampleLength);
	  ZeroMemory(MF->asioDriverInfo.bufferInfos[MF->RChO].buffers[bufindex], len * MF->asioSampleLength);
	}
	else
	{
	  MF->OLring->get((char *)MF->asioDriverInfo.bufferInfos[MF->LChO].buffers[bufindex], len);
	  MF->ORring->get((char *)MF->asioDriverInfo.bufferInfos[MF->RChO].buffers[bufindex], len);
	}

	if(MF->asioDriverInfo.postOutput) ASIOOutputReady();
// ------------------------------------------------------------

	if(!MainForm->wavin)
	{
	  pL = (char *)MF->asioDriverInfo.bufferInfos[MF->LChI].buffers[bufindex];
	  pR = (char *)MF->asioDriverInfo.bufferInfos[MF->RChI].buffers[bufindex];
	}
	else   // if reading from wav file...should never happen, for wav only WMME is used...
	{
	  MainForm->PIn->wreadlen = len;
	  MainForm->IWav->InRead(
		   MainForm->PIn->inL, MainForm->PIn->inR, MainForm->PIn->wreadlen);

	  MainForm->EoF = MainForm->PIn->wreadlen != len;
	  for(int k = 0; k < len; k++)
	  {
		tL32[k] = MainForm->PIn->inL[k] << 8;
		tR32[k] = MainForm->PIn->inR[k] << 8;
	  }

	  pL = tL32;
	  pR = tR32;
	}

// from card (or wav file) to ring ----------------------------

	MF->ILring->put((char *)pL, len);
	MF->IRring->put((char *)pR, len);
// ------------------------------------------------------------


// Now see if it is the right time to awake the processing thread
	int curlen = MF->ILring->getlen();  // # of samples accumulated so far

	if(curlen >= BUFSIZE)  // at least one buffer worth of samples to process
	{
	  MF->asioINfilled = 0;
	  while((curlen -= BUFSIZE) >= 0) MF->asioINfilled++;
	  SetEvent(MF->InBufferReady);
	}

	return 0L;
}
//---------------------------------------------------------------------------
void __fastcall getASIOsamples(int *inL32, int *inR32)
{
// from ring to program ---------------------------------------

	  MF->ILring->get((char *)inL32, BUFSIZE);
	  MF->IRring->get((char *)inR32, BUFSIZE);

	  switch(MF->asioDriverInfo.channelInfos[0].type) //let's suppose the sample
	  {                                               //type is the same on all channels...
		case ASIOSTInt32LSB :
		  break;

		case ASIOSTInt24LSB :
		  MF->ASIOConv->int24to32inPlace(inL32, BUFSIZE);
		  MF->ASIOConv->int24to32inPlace(inR32, BUFSIZE);
		  break;

		case ASIOSTInt16LSB :
		  MF->ASIOConv->int16to32inPlace(inL32, BUFSIZE);
		  MF->ASIOConv->int16to32inPlace(inR32, BUFSIZE);
		  break;
	  }
}
//---------------------------------------------------------------------------
void __fastcall putASIOsamples(int *outL32, int *outR32)
{
// from program to ring ----------------------------------------

	  switch(MF->asioDriverInfo.channelInfos[0].type) //let's suppose the sample
	  {                                               //type is the same on all channels...
		case ASIOSTInt32LSB :
		  break;

		case ASIOSTInt24LSB :
		  MF->ASIOConv->int32to24inPlace(outL32, BUFSIZE);
		  MF->ASIOConv->int32to24inPlace(outR32, BUFSIZE);
		  break;

		case ASIOSTInt16LSB :
		  MF->ASIOConv->int32to16inPlace(outL32, BUFSIZE);
		  MF->ASIOConv->int32to16inPlace(outR32, BUFSIZE);
		  break;
	  }

	  MF->OLring->put((char *)outL32, BUFSIZE);
	  MF->ORring->put((char *)outR32, BUFSIZE);

}
//---------------------------------------------------------------------------
#pragma package(smart_init)

