#ifndef DefinesH
  #define DefinesH

  #define nsp_UsesVector
  #define nsp_UsesConversion
  #define nsp_UsesWin
  #define nsp_UsesLms
  #define nsp_UsesTransform
  #define nsp_UsesMisc
  #define nsp_UsesFir
  #define nsp_UsesIir
  #define nsp_UsesFirGen
//  #define nsp_UsesAll

//--------------------------------------------------------------------------
//
//  Define:         Prevent inclusion of:
//  --------------  --------------------------------------------------------
//  MMNODRV         Installable driver support
//  MMNOSOUND       Sound support
//  MMNOWAVE        Waveform support
//  MMNOMIDI        MIDI support
//  MMNOAUX         Auxiliary audio support
//  MMNOMIXER       Mixer support
//  MMNOTIMER       Timer support
//  MMNOJOY         Joystick support
//  MMNOMCI         MCI support
//  MMNOMMIO        Multimedia file I/O support
//  MMNOMMSYSTEM    General MMSYSTEM functions
//
  #define MMNODRV
  #define MMNOMIDI
  #define MMNOAUX
//  #define MMNOTIMER
  #define MMNOJOY
  #define MMNOMCI
  #define MMNOMMIO
//--------------------------------------------------------------------------

  #define DEFAULT_RATE  48000
  #define BUFSIZE   (512 * 2)
  #define OUTBUFS   (512 * 2)
  #define OUTBUFFERMARK 5
  #define RINGSIZE  (32768 * 2)
  #define FF1SMIN   1024
  #define FF1SMAX   (FF1SMIN * 32)
  #define FF2SMIN   1024
  #define FF2SMAX   (FF2SMIN * 64)
  #define POLYFAC   4
  #define NINBUF    (64 * 32)
  #define NOUTBUF   (64 * 32)
  #define BYTEIN    2
  #define BYTEOUT   2
  #define AVGFF1    2
  #define AVGFF2    1
  #define ERRLEN    120
  #define SINCLEN   100000
  #define SINCSTEP  (SINCLEN/10)

  #define SINELEN    (256 * 1024)
  #define AFFTLEN    (BUFSIZE * 4)
  #define PREFFTLEN  (BUFSIZE * 2)
  #define LPFIRLEN    256
  #define LPFIRUPLEN  144
  #define PMLPLENW    256
  #define PMLPLENN    256

  #define DOWN24     24
  #define DOWN48     48
  #define MAXDOWN    DOWN48
  #define ALLPASSLENGTH 127
  #define LLEN      (ALLPASSLENGTH + 2)
  #define RLEN      (ALLPASSLENGTH + 1)
  #define NUMDIGITS 9
  #define NUMRATES  12

  #define PI         3.14159265358979323846
  #define twoPI      (2. * PI)
  #define fourPI     (2. * twoPI)
  #define sixPI      (3. * twoPI)
  #define PIhalf     (PI / 2.)
  #define PIquart    (PI / 4.)

  #define twoTO24    (16777216.)
  #define twoTO23    (8388608.)
  #define twoTO23m1  (8388607.)
  #define twoTO22    (4194304.)
  #define twoTOplus8 (256.)
  #define NORMCONST  (1./twoTO23)
  #define NORMCONSTP (NORMCONST)
  #define ONEFIFTH   (1.F/5.F)
  #define ONETHIRD   (1.F/3.F)
  #define myNavy      ((TColor) 0x00a00000)
  #define clLevelBack ((TColor) 0x00C8B5A6)
  #define clDim       ((TColor) 0x00D5C9C0)

  #define clButtOff   ((TColor) 0x00152615)
  #define clButtOn    ((TColor) 0x000BD200)
  #define clTextOff   ((TColor) 0x0083B1E0)
  #define clTextOn    clNavy
  #define clPesca     ((TColor) 0x0083B1E0)
  #define clLight     ((TColor) 0x00E3DFE0)
//  #define clPesca     ((TColor) 0x007BD5F2)

  #define KEY_LEN   1024
  #define           VERSION  "1.0"

  #define MF    MainForm
  #define min(a,b)  ((a) > (b) ? (b) : (a))
  #define max(a,b)  ((a) > (b) ? (a) : (b))

  typedef short AudioInBuffers [NINBUF+1][BUFSIZE * 2];  // two channels
  typedef short AudioOutBuffers[NOUTBUF+1][OUTBUFS *2];  // two channels

  typedef enum  {Hann, Blackman, Hamming, wsin3, wsin5, nuttall, nowin} wintype;
  typedef enum  {darker, Spectran, lighter, Horne, SpecExt, Winrad, Linrad} MyPalette;
  typedef enum  {off, fast, slow} agc;
  typedef enum  {WMME, ASIO, mustStop} driver;
  typedef enum  {mLeft, mRight, mLeftRight, mIQ} datamode;
  typedef enum  {AM, ECSS, FM, LSB, USB, CW, DRM, NONE} runmode;
  typedef enum  {lower, upper, both} submode;
  typedef enum  {none, SDR14, SDRX, USBdata16, SCdata, USBdata24, USBdata32, USBfloat32}
				extHWtype;
  typedef enum  {wstopped, wpaused, wrecording} wrec;				

  typedef bool  (__stdcall * pfnGetSystemTimes)(__int64* lpIdleTime,
							  __int64* lpKernelTime, __int64* lpUserTime );

  typedef bool  (__stdcall * pfnInitHW)(char *name, char *model, int& index);
  typedef bool  (__stdcall * pfnOpenHW)(void);
  typedef void  (__stdcall * pfnCloseHW)(void);
  typedef int   (__stdcall * pfnStartHW)(int extLOfreq);
  typedef void  (__stdcall * pfnStopHW)(void);
  typedef void  (__stdcall * pfnSetCallback)(void (*) (int, int, float, void *));
  typedef int   (__stdcall * pfnSetHWLO)(long);
  typedef long  (__stdcall * pfnGetHWLO)(void);
  typedef int   (__stdcall * pfnGetStatus)(void);
  typedef long  (__stdcall * pfnGetHWSR)(void);
  typedef void  (__stdcall * pfnRawDataReady)(long samprate, void *Ldata, void *Rdata, int numsamples);
  typedef void  (__stdcall * pfnShowGUI)(void);
  typedef void  (__stdcall * pfnHideGUI)(void);
  typedef void  (__stdcall * pfnTuneChanged)(long tunefreq);
  typedef long  (__stdcall * pfnGetTune)(void);
  typedef void  (__stdcall * pfnModeChanged)(char mode);
  typedef char  (__stdcall * pfnGetMode)(void);
  typedef void  (__stdcall * pfnIFLimitsChanged)(long lowfreq, long highfreq);

  typedef void  (__stdcall * pfnFiltersChanged)
								  (int loCut, int hiCut, int pitch);

  typedef void  (__stdcall * pfnGetFilters)
							 (int& loCut, int& hiCut, int& pitch);

  struct  WinradStatus
  {
	 int     srateIn, GainPos1, BasePos1, WF1GainPos, WF1ContrastPos, WF2ContrastPos;
	 int     AGCGainPos, GainPos2, BasePos2, WF2GainPos, WF1SpeedPos, WF2SpeedPos;
	 int     WF1ResPos, CWWF2Resol, SSBWF2Resol, AvgIdx1, AvgIdx2, polyfactor;
	 int     rightdly, leftdly, FMWF2Resol, DRMWF2Resol;
	 float   Den1fac, vol, Den2LvlPos, BlkThr, sampleRate;  // used for ASIO
	 float   SSBFF2locut, SSBFF2hicut, FF1freq, CWPitch, CWbw, skew, Qmult;
	 float   extLOfreq, extLOoffset, FMFF2locut, FMFF2hicut, DRMFF2locut, DRMFF2hicut;
	 float   wfm_deemph_tau;
	 int	 wfm_rf_cut, wfm_audio_cut;
	 char    wavdir[256];
	 char    wavname[256];
	 bool    nr1, nr2, nblanker, swapIQ, wf1slow, sp1fullspeed, revWF1scroll;
	 bool    WF1avg, WF2avg, vollocked;
	 DWORD   procprio;
	 wintype wintype;
	 MyPalette activePalette;
	 agc       agc;
	 driver    driver;
	 datamode  datamode;
	 runmode   mode;
  };

  struct  MYPAL
  { WORD         palVersion;
	WORD         palNumEntries;
	PALETTEENTRY palPalEntry[256];
  };

  struct riffhdr
  {
		char riff[4];      // RIFF string
		long filesize;     // filesize (maybe with some byte missing...)
		char wave[4];      // WAVE string
  };

  struct fmthdr
  {
		char   ckID[4];
		long   nChunkSize;
		short  wFormatTag;
		short  nChannels;
		long   nSamplesPerSec;
		long   nAvgBytesPerSec;
		short  nBlockAlign;
		short  nBitsPerSample;
  };

  struct datahdr
  {
		char ckID[4];                 
		long nChunkSize;
  };

// Formato chunk "rcrv" di Perseus

  struct rcvrhdr
  {
	char  		chunkID[4];	          // ="rcvr" (chunk perseus beta0.2)
	long  		chunkSize;	          // lunghezza del chunk
	long		nCenterFrequencyHz;   // Frequenza centrale di acquisizione
	long 		SamplingRateIdx;      // 0=125K, 1=250K, 2=500K
	time_t 		timeStart;	          // Ora inizio registrazione (ottenuta con time(&timeStart))
	unsigned short 	wAttenId;	      // 0=0dB, 1=10dB, 2=20dB, 3=30dB
	char		bAdcPresel;	          // 0=Presel Off, 1=Presel On
	char		bAdcPreamp;	          // 0=ADC Preamp Off, 1=ADC Preamp ON
	char		bAdcDither;	          // 0=ADC Dither Off, 1=ADC Dither ON
	char		bSpare;		          // for future use (default = 0)
	char        rsrvd[16];	          // for future use (default = 000..0)
  };

  struct auxihdr     // used by SpectraVue in WAV files
  {
	char  	   chunkID[4];	          // ="auxi" (chunk rfspace)
	long  	   chunkSize;	          // lunghezza del chunk
	SYSTEMTIME StartTime;
	SYSTEMTIME StopTime;
	DWORD CenterFreq; //receiver center frequency
	DWORD ADFrequency; //A/D sample frequency before downsampling
	DWORD IFFrequency; //IF freq if an external down converter is used
	DWORD Bandwidth; //displayable BW if you want to limit the display to less than Nyquist band
	DWORD IQOffset; //DC offset of the I and Q channels in 1/1000's of a count
	DWORD Unused2;
	DWORD Unused3;
	DWORD Unused4;
	DWORD Unused5;
	char  nextfilename[96];
  };




#endif
